"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Kernel = void 0;
/**
 * @module Kernel
 */
const awilix_1 = require("awilix");
const awilix_manager_1 = require("awilix-manager");
const koa_1 = __importDefault(require("koa"));
const koa_qs_1 = __importDefault(require("koa-qs"));
const node_util_1 = require("node:util");
const routing_1 = require("./decorators/routing");
const middleware_1 = require("./middleware");
const authenticateAnonymous_1 = require("./middleware/authenticateAnonymous");
/**
 * The kernel class is the orchestration layer of your application.
 * It serves as the entry point that initializes the container, prepares the Koa application, wires up middleware, and loads your controller routes.
 *
 */
class Kernel {
    constructor(options) {
        this.options = options;
        this.logger = this.createLogger();
        this.container = this.createContainer();
        this.app = (0, koa_qs_1.default)(this.createApp());
    }
    get server() {
        return this._server;
    }
    get address() {
        return this._address;
    }
    async boot() {
        await this.buildContainer();
        await (0, awilix_manager_1.asyncInit)(this.container);
        this.addMiddlewares();
        this.loadControllers();
    }
    async serve() {
        const host = this.options.get('host');
        const port = this.options.get('port');
        const serving = new Promise((Ok) => {
            this._server = this.app.listen(port, host, () => {
                this._address = this._server.address();
                this.logger.info('Server running', { host, port, url: `http://${host}:${port}` });
                Ok(this._server);
            });
        });
        return await serving;
    }
    async shutdown() {
        await (0, awilix_manager_1.asyncDispose)(this.container);
        if (true === Boolean(this._server)) {
            await (0, node_util_1.promisify)(this._server.close).call(this._server);
        }
    }
    createContainer() {
        return (0, awilix_1.createContainer)({ injectionMode: 'CLASSIC', strict: true });
    }
    async buildContainer() {
        const builder = this.createContainerBuilder();
        await builder.build(this.container);
    }
    addMiddlewares() {
        this.app.use((0, middleware_1.errorHandler)(this.logger));
        this.app.use((0, authenticateAnonymous_1.authenticateAnonymous)());
        this.app.use((0, middleware_1.containerScope)(this.container));
        for (const middleware of this.middlewares()) {
            this.app.use(middleware);
        }
    }
    createApp() {
        return new koa_1.default();
    }
    middlewares() {
        return [];
    }
    loadControllers() {
        const prefix = this.options.get('namespace');
        const router = (0, routing_1.mountControllers)(prefix);
        this.app.use(router.routes());
    }
}
exports.Kernel = Kernel;
//# sourceMappingURL=kernel.js.map