"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LocalMessageBus = exports.LazyEventHandler = void 0;
class LazyEventHandler {
    constructor(spec, serviceKey, container) {
        this.spec = spec;
        this.serviceKey = serviceKey;
        this.container = container;
    }
    get eventName() {
        return this.spec.eventName;
    }
    get eventVersion() {
        return this.spec.eventVersion;
    }
    usingMessageBus(owner) {
        this.messageBus = owner;
    }
    async handleEvent(event) {
        const actualHandler = this.container.resolve(this.serviceKey);
        actualHandler.usingMessageBus(this.messageBus);
        await actualHandler.handleEvent(event);
    }
}
exports.LazyEventHandler = LazyEventHandler;
class LocalMessageBus {
    constructor(eventHandlers = [], appLogger) {
        this.eventHandlers = new Map();
        this.logger = appLogger;
        for (let handler of eventHandlers) {
            this.subscribeToEvent(handler);
        }
    }
    async initialize() { }
    async dispose() { }
    async publishEvent(event) {
        const versionedHandlers = this.eventHandlers.get(event.name)?.get(event.version);
        const eventInfo = { eventName: event.name, eventVersion: event.version };
        if (!versionedHandlers || versionedHandlers.size === 0) {
            this.logger.debug('DomainEventUnhandled', eventInfo);
            return false;
        }
        try {
            await Promise.all([...versionedHandlers].map((handler) => handler.handleEvent(event)));
            this.logger.debug('DomainEventHandled', eventInfo);
        }
        catch (error) {
            this.logger.error('DomainEventError', { ...eventInfo, error });
            // throw error
        }
        return true;
    }
    subscribeToEvent(eventHandler) {
        if (!this.eventHandlers.has(eventHandler.eventName)) {
            this.eventHandlers.set(eventHandler.eventName, new Map());
        }
        const versionMap = this.eventHandlers.get(eventHandler.eventName);
        if (!versionMap.has(eventHandler.eventVersion)) {
            versionMap.set(eventHandler.eventVersion, new Set());
        }
        eventHandler.usingMessageBus(this);
        versionMap.get(eventHandler.eventVersion).add(eventHandler);
        this.logger.debug('EventHandlerSubscribed', this.handlerInfo(eventHandler));
    }
    handlerInfo(eventHandler) {
        return {
            eventName: eventHandler.eventName,
            eventVersion: eventHandler.eventVersion,
            handlerName: eventHandler.constructor.name,
        };
    }
    unsubscribeFromEvent(eventHandler) {
        const versionMap = this.eventHandlers.get(eventHandler.eventName);
        if (!versionMap)
            return;
        const handlerSet = versionMap.get(eventHandler.eventVersion);
        if (!handlerSet)
            return;
        handlerSet.delete(eventHandler);
        // Clean up empty sets/maps
        if (handlerSet.size === 0)
            versionMap.delete(eventHandler.eventVersion);
        if (versionMap.size === 0)
            this.eventHandlers.delete(eventHandler.eventName);
        this.logger.debug('EventHandlerUnsubscribed', this.handlerInfo(eventHandler));
    }
}
exports.LocalMessageBus = LocalMessageBus;
//# sourceMappingURL=messageBus.js.map