"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractDocumentor = void 0;
const oas_builder_1 = __importDefault(require("@avanzu/oas-builder"));
const typebox_1 = require("@sinclair/typebox");
const http_status_codes_1 = require("http-status-codes");
const node_path_1 = __importDefault(require("node:path"));
const docs_1 = require("../../decorators/docs");
const errors_1 = require("../../errors");
function ErrorSchema(statusCode) {
    return typebox_1.Type.Object({
        errorCode: typebox_1.Type.Enum(errors_1.ErrorCodes),
        statusCode: typebox_1.Type.Literal(statusCode),
        reason: typebox_1.Type.Literal((0, http_status_codes_1.getReasonPhrase)(statusCode)),
        message: typebox_1.Type.String(),
        details: typebox_1.Type.Record(typebox_1.Type.String(), typebox_1.Type.Any()),
    });
}
class AbstractDocumentor {
    createRoute(context) {
        const route = node_path_1.default.join(context.info.namespace || '', context.mountpoint.prefix || '', context.endpoint.route);
        return route.replace(/:([^\/]+)/g, '{$1}');
    }
    createOperationId(context) {
        const methodName = context.endpoint.method.toString();
        const controllerClassName = context.endpoint.target.prototype.constructor.name;
        const controllerName = controllerClassName.replace(/Controller$/, '');
        const firstCapital = methodName.match(/[A-Z]/)?.index || -1;
        return `${methodName.slice(0, firstCapital)}${controllerName}${methodName.slice(firstCapital)}`;
    }
    addQuerySchema(operation, context) {
        const query = (0, docs_1.getSimpleQuerySchema)(context.endpoint.target, context.endpoint.method);
        if (null == query)
            return operation;
        const schema = query();
        for (const [name, type] of Object.entries(schema)) {
            operation = operation.parameter(oas_builder_1.default.param().inQuery().name(name).schema(type));
        }
        return operation;
    }
    addInfo(operation, context) {
        const info = (0, docs_1.getInfo)(context.endpoint.target, context.endpoint.method);
        if (null == info)
            return operation;
        const { summary, description } = info();
        return operation.summary(summary).description(description);
    }
    addParams(operation, context) {
        const schema = (0, docs_1.getParamSchema)(context.endpoint.target, context.endpoint.method);
        if (null == schema)
            return operation;
        for (const [name, type] of Object.entries(schema())) {
            operation = operation.parameter(oas_builder_1.default.param().inPath().name(name).schema(type));
        }
        return operation;
    }
    addQuery(operation, context) {
        const query = (0, docs_1.getQuerySchema)(context.endpoint.target, context.endpoint.method);
        if (null == query)
            return operation;
        const schemaProvider = context.container.build(query());
        for (const [name, type] of Object.entries(schemaProvider)) {
            operation = operation.parameter(oas_builder_1.default.param().inQuery().name(name).schema(type));
        }
        return operation;
    }
    addRequestSchema(operation, context) {
        const schema = (0, docs_1.getRequestSchema)(context.endpoint.target, context.endpoint.method);
        if (null == schema)
            return operation;
        const content = oas_builder_1.default.content().schema(schema());
        const body = oas_builder_1.default.body().json(content);
        return operation.request(body);
    }
    addResponseSchema(operation, context) {
        const schema = (0, docs_1.getResponseSchema)(context.endpoint.target, context.endpoint.method);
        if (null == schema)
            return operation;
        const schemValue = schema();
        for (const [status, definition] of Object.entries(schemValue)) {
            const content = oas_builder_1.default.content().schema(definition);
            const body = oas_builder_1.default.body().description((0, http_status_codes_1.getReasonPhrase)(status)).json(content);
            operation = operation.response(status, body);
        }
        return operation;
    }
    addErrorCodes(operation, context) {
        const codes = (0, docs_1.getErrorCodes)(context.endpoint.target, context.endpoint.method);
        if (null == codes)
            return operation;
        const codeValues = codes();
        for (const status of codeValues) {
            const content = oas_builder_1.default.content().schema(ErrorSchema(status));
            const body = oas_builder_1.default.body().description((0, http_status_codes_1.getReasonPhrase)(status)).json(content);
            operation = operation.response(status, body);
        }
        return operation;
    }
    addErrorSchema(operation, context) {
        const schema = (0, docs_1.getErrorSchema)(context.endpoint.target, context.endpoint.method);
        if (null == schema)
            return operation;
        const schemValue = schema();
        for (const [status, definition] of Object.entries(schemValue)) {
            const content = oas_builder_1.default.content().schema(definition);
            const body = oas_builder_1.default.body().description((0, http_status_codes_1.getReasonPhrase)(status)).json(content);
            operation = operation.response(status, body);
        }
        return operation;
    }
    addTag(operation, context) {
        if (null == context.opts.tag)
            return operation;
        return operation.tag(context.opts.tag.name);
    }
}
exports.AbstractDocumentor = AbstractDocumentor;
//# sourceMappingURL=abstractDocumentor.js.map